// src/components/AppointmentForm.jsx
import React, { useState, useEffect } from 'react';
import Modal from 'react-modal';
import moment from 'moment-jalaali';
import { createAppointment } from '../services/api';

Modal.setAppElement('#root');

export default function AppointmentForm({
  isOpen,
  onRequestClose,
  onCreated,
  patientsList,
}) {
  const [form, setForm] = useState({
    patientId: '',
    type: 'Injection',
    dateShamsi: moment().format('jYYYY-jMM-jDD'),
    time: '',
    status: 'Scheduled',
  });

  const handleChange = e => {
    const { name, value } = e.target;
    setForm(f => ({ ...f, [name]: value }));
  };

  const handleSubmit = async e => {
    e.preventDefault();
    const { data } = await createAppointment(form);
    onCreated(data);
    onRequestClose();
  };

  return (
    <Modal
      isOpen={isOpen}
      onRequestClose={onRequestClose}
      className="bg-white p-4 rounded shadow max-w-md mx-auto mt-20"
      overlayClassName="fixed inset-0 bg-black bg-opacity-40"
    >
      <h2 className="text-xl font-bold mb-4">ثبت نوبت جدید</h2>
      <form onSubmit={handleSubmit} className="space-y-3">
        <select
          name="patientId"
          value={form.patientId}
          onChange={handleChange}
          className="w-full border p-2 rounded"
          required
        >
          <option value="">انتخاب بیمار</option>
          {patientsList.map(p => (
            <option key={p._id} value={p._id}>
              {p.fullName} ({p.phone})
            </option>
          ))}
        </select>

        <select
          name="type"
          value={form.type}
          onChange={handleChange}
          className="w-full border p-2 rounded"
        >
          <option value="Injection">تزریق</option>
          <option value="Laser">لیزر</option>
        </select>

        <input
          name="dateShamsi"
          type="text"
          value={form.dateShamsi}
          onChange={handleChange}
          className="w-full border p-2 rounded"
          placeholder="تاریخ شمسی (مثلاً 1402-12-30)"
        />

        <input
          name="time"
          placeholder="ساعت (اختیاری، مثلاً 15:30)"
          value={form.time}
          onChange={handleChange}
          className="w-full border p-2 rounded"
        />

        <button
          type="submit"
          className="w-full bg-blue-500 text-white py-2 rounded"
        >
          ذخیره نوبت
        </button>
      </form>
    </Modal>
  );
}
