// src/components/PatientForm.jsx
import React, { useState, useEffect } from 'react';
import Modal from 'react-modal';
import { createPatient, updatePatient } from '../services/api';

Modal.setAppElement('#root');

export default function PatientForm({
  isOpen,
  onRequestClose,
  onCreated,
  onUpdated,
  patientToEdit,    // ← اضافه شد
}) {
  const initialState = {
    fullName: '',
    birthDate: '',
    phone: '',
    address: '',
    tag: '',
    photos: [],
  };
  const [form, setForm] = useState(initialState);
  const [error, setError] = useState(null);

  // هر بار که patientToEdit تغییر کرد، فرم را پر کن
  useEffect(() => {
    if (patientToEdit) {
      setForm({
        fullName:    patientToEdit.fullName  || '',
        birthDate:   patientToEdit.birthDate 
                      ? patientToEdit.birthDate.slice(0,10) 
                      : '',
        phone:       patientToEdit.phone     || '',
        address:     patientToEdit.address   || '',
        tag:         patientToEdit.tag       || '',
        photos:      patientToEdit.photos    || [],
      });
    } else {
      setForm(initialState);
    }
  }, [patientToEdit]);

  const handleChange = e => {
    const { name, value } = e.target;
    setForm(f => ({ ...f, [name]: value }));
  };

  const handleSubmit = async e => {
    e.preventDefault();
    try {
      if (patientToEdit && patientToEdit._id) {
        // ویرایش
        const { data } = await updatePatient(patientToEdit._id, form);
        onUpdated(data);
      } else {
        // افزودن جدید
        const { data } = await createPatient(form);
        onCreated(data);
      }
      onRequestClose();
      setForm(initialState);
    } catch (err) {
      setError(err.response?.data?.error || err.message);
    }
  };

  return (
    <Modal
      isOpen={isOpen}
      onRequestClose={() => {
        onRequestClose();
        setError(null);
      }}
      className="bg-white p-4 rounded shadow max-w-md mx-auto mt-20"
      overlayClassName="fixed inset-0 bg-black bg-opacity-50"
    >
      <h2 className="text-xl font-bold mb-4">
        {patientToEdit ? 'Edit Patient' : 'Add Patient'}
      </h2>
      {error && (
        <div className="text-red-500 mb-2">
          {typeof error === 'string' ? error : JSON.stringify(error)}
        </div>
      )}
      <form onSubmit={handleSubmit} className="space-y-2">
        <input
          name="fullName"
          placeholder="Full Name"
          value={form.fullName}
          onChange={handleChange}
          className="w-full border p-2 rounded"
          required
        />
        <input
          name="birthDate"
          type="date"
          placeholder="Birth Date"
          value={form.birthDate}
          onChange={handleChange}
          className="w-full border p-2 rounded"
        />
        <input
          name="phone"
          placeholder="Phone"
          value={form.phone}
          onChange={handleChange}
          className="w-full border p-2 rounded"
          required
        />
        <input
          name="address"
          placeholder="Address"
          value={form.address}
          onChange={handleChange}
          className="w-full border p-2 rounded"
        />
        <input
          name="tag"
          placeholder="Tag"
          value={form.tag}
          onChange={handleChange}
          className="w-full border p-2 rounded"
        />
        {/* عکس‌ها را اینجا اضافه نکنیم */}
        <div className="flex justify-end space-x-2 pt-4">
          <button
            type="button"
            onClick={() => {
              onRequestClose();
              setError(null);
            }}
            className="px-4 py-2 border rounded"
          >
            Cancel
          </button>
          <button type="submit" className="px-4 py-2 bg-blue-500 text-white rounded">
            Save
          </button>
        </div>
      </form>
    </Modal>
  );
}
