// src/pages/Appointments.jsx
import React, { useEffect, useState } from 'react';
import { fetchAppointments, fetchPatients } from '../services/api';
import AppointmentForm from '../components/AppointmentForm';

export default function Appointments() {
  const [appointments, setAppointments] = useState([]);
  const [patients, setPatients] = useState([]);
  const [loading, setLoading] = useState(true);
  const [showForm, setShowForm] = useState(false);

  const load = async () => {
    setLoading(true);
    const resPatients = await fetchPatients();
    const resAppointments = await fetchAppointments();
    setPatients(resPatients.data);
    setAppointments(resAppointments.data);
    setLoading(false);
  };

  useEffect(() => {
    load();
  }, []);

  const handleCreated = newAppt => {
    setAppointments(a => [newAppt, ...a]);
  };

  return (
    <div>
      <div className="flex justify-between items-center mb-4">
        <h1 className="text-2xl font-bold">Appointments</h1>
        <button
          onClick={() => setShowForm(true)}
          className="bg-green-500 text-white px-4 py-2 rounded"
        >
          Add Appointment
        </button>
      </div>

      {loading ? (
        <p>Loading…</p>
      ) : (
        <table className="min-w-full bg-white shadow rounded">
          <thead>
            <tr>
              <th className="px-4 py-2">Date</th>
              <th className="px-4 py-2">Patient</th>
              <th className="px-4 py-2">Type</th>
              <th className="px-4 py-2">Time</th>
            </tr>
          </thead>
          <tbody>
            {appointments.map(appt => (
              <tr key={appt._id} className="border-t">
                <td className="px-4 py-2">{appt.dateShamsi}</td>
                <td className="px-4 py-2">{appt.patientId?.fullName || '—'}</td>
                <td className="px-4 py-2">{appt.type}</td>
                <td className="px-4 py-2">{appt.time || '—'}</td>
              </tr>
            ))}
          </tbody>
        </table>
      )}

      <AppointmentForm
        isOpen={showForm}
        onRequestClose={() => setShowForm(false)}
        onCreated={handleCreated}
        patientsList={patients}
      />
    </div>
  );
}
